using Frontend;
using Frontend.AssetStores;
using Frontend.BackgroundServices;
using Frontend.Marketplaces;
using Frontend.Marketplaces.Interfaces;
using Frontend.Metrics;
using Microsoft.EntityFrameworkCore;
using OpenTelemetry.Metrics;
using OpenTelemetry.Resources;

// the location differ in dev and releas mode.
var tokenAbiLocation = Environment.GetEnvironmentVariable("TOKEN_ABI_LOCATION")!;
var tokenAbi = File.ReadAllText(tokenAbiLocation)!;

var chainApiUrl = Environment.GetEnvironmentVariable("API_URL")!;
var accountPrivateKey = Environment.GetEnvironmentVariable("ACCOUNT_PRIVATE_KEY")!;

var postgresHost = Environment.GetEnvironmentVariable("POSTGRES_HOST")!;
var postgresUser = Environment.GetEnvironmentVariable("POSTGRES_USER")!;
var postgresPassword = Environment.GetEnvironmentVariable("POSTGRES_PASSWORD")!;
var postgresDb = Environment.GetEnvironmentVariable("POSTGRES_DB")!;

var builder = WebApplication.CreateBuilder(args);

// Configurations
var chainConfiguration = builder.Configuration.GetSection("Chain");

// Add services to the container.
builder.Services.AddHostedService<PrometheusService>();
builder.Services.AddSingleton<Erc20TokenMetrics>();
builder.Services.AddSingleton<EthMetrics>();
builder.Services.AddSingleton<PotatoStorageMetrics>();
builder.Services.AddSingleton<PotatoMarketplaceMetrics>();
builder.Services.AddSingleton<IPotatoMarketPlace>(sp => new PotatoMarketplaceMock());

builder.Services.AddSingleton(provider => new TokenClient(
    chainApiUrl,
    chainConfiguration["tokenAddress"]!,
    tokenAbi,
    accountPrivateKey
));

builder.Services.AddDbContextFactory<PotatoStorage>(options =>
    options.UseNpgsql($"Host={postgresHost};Database={postgresDb};Username={postgresUser};Password={postgresPassword}"));

builder.Services.AddRazorPages();
builder.Services.AddOpenTelemetry()
    .WithMetrics(builder =>
    {
        builder.AddPrometheusExporter();

        builder.AddMeter("Microsoft.AspNetCore.Hosting",
                         "Microsoft.AspNetCore.Server.Kestrel",
                         "RoestiCoin");
    });

var app = builder.Build();

var potatoStorageFactory = app.Services.GetRequiredService<IDbContextFactory<PotatoStorage>>();
var potatoStorage = potatoStorageFactory.CreateDbContext();
potatoStorage.Database.EnsureCreated();

// Configure the HTTP request pipeline.
if (!app.Environment.IsDevelopment())
{
    app.UseExceptionHandler("/Error");
    // The default HSTS value is 30 days. You may want to change this for production scenarios, see https://aka.ms/aspnetcore-hsts.
    app.UseHsts();

    app.UseHttpsRedirection();
}

app.MapPrometheusScrapingEndpoint();
app.UseStaticFiles();

app.UseRouting();

app.UseAuthorization();

app.MapRazorPages();

app.Run();
